<?php
namespace Aelia\WC\CurrencySwitcher\AffiliateWP;
if(!defined('ABSPATH')) exit; // Exit if accessed directly

use Aelia\WC\CurrencySwitcher\AffiliateWP\Order as Order;

/**
 * Implements the integration for the AffiliateWP plugin.
 *
 * @since 1.0.1.150518
 */
class AffiliateWP_Integration {
	protected static $_instance;

	/**
	 * Returns an instance of the class, creating one on the fly if needed.
	 *
	 * @return AffiliateWP_Integration
	 */
	public static function instance() {
		if(empty(self::$_instance)) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	/**
	 * Returns the instance of the logger used by the plugin.
	 *
	 * @return \Aelia\WC\Logger.
	 * @since 1.0.12.200505
	 */
	protected function get_logger() {
		return Aelia_CS_AffiliateWP::instance()->get_logger();
	}

	/**
	 * Constructor.
	 */
	public function __construct() {
		$this->set_hooks();
	}

	/**
	 * Set the hooks required by the class.
	 */
	protected function set_hooks() {
		// Price conversion
		add_filter('affwp_get_woocommerce_order', array($this, 'affwp_get_woocommerce_order'), 10, 2);

		// Add hooks for AffiliateWP - Recurring Referrals
		// @since 1.0.7.180104
		$this->set_recurring_referrals_hooks();
	}

	/**
	 * Adds hooks related to AffiliateWP -  Recurring referrals.
	 *
	 * @since 1.0.7.180104
	 */
	protected function set_recurring_referrals_hooks() {
		add_filter('affwp_recurring_create_referral', array($this, 'affwp_recurring_create_referral'), 1, 2);
	}

	/**
	 * Processes an order object before it's used by AffiliateWP. This method
	 * replaced an order instance with an instance of
	 * \Aelia\WC\CurrencySwitcher\AffiliateWP\Order class, which will return the
	 * order totals in the currency used by Affiliate WP.
	 *
	 * @param WC_Order order The order being processed by AffiliateWP.
	 * @return \Aelia\WC\CurrencySwitcher\AffiliateWP\Order An order class,
	 * @see \Aelia\WC\CurrencySwitcher\AffiliateWP\Order
	 */
	public function affwp_get_woocommerce_order($order) {
		// Replace the order instance with an extended order class, which will
		// return the totals in the currency expected by AffiliateWP
		$order_in_awp_currency = Order::factory(aelia_get_order_id($order));

		// Debug logging
		// @since 1.0.12.200505
		$this->get_logger()->debug(__('Converted order totals before commission calculation.', 'wc-aelia-eu-vat-assistant'), array(
			'Order ID' => $order->get_id(),
			'Original Order Currency' => $order->get_currency(),
			'AffiliateWP Currency' => affwp_get_currency(),
		));

		return $order_in_awp_currency;
	}

	/**
	 * Converts an amount to the AffiliateWP currency.
	 *
	 * @param float amount
	 * @param string currency
	 * @return float
	 * @since 1.0.7.180104
	 */
	protected function convert_to_awp_currency($amount, $source_currency) {
		$original_amount = $amount;
		$amount = apply_filters('wc_aelia_cs_convert',
														$original_amount,
														// Source currency - This order's currency
														$source_currency,
														// Destination currency - AffiliateWP currency
														affwp_get_currency());

		// Debug logging
		// @since 1.0.12.200505
		$this->get_logger()->debug(__('Converted amount to AWP currency.', 'wc-aelia-eu-vat-assistant'), array(
			'Original Amount' => $original_amount,
			'Converted Amount' => $amount,
			'Source Currency' => $source_currency,
			'AffiliateWP Currency' => affwp_get_currency(),
		));

		return apply_filters('wc_aelia_cs_affiliatewp_convert_amount', $amount, $this);
	}

	/**
	 * Converts a recurring referral amount to the AffiliateWP currency, before
	 * the referral is created.
	 *
	 * @param float referral_amount
	 * @param string affiliate_id
	 * @param float order_amount
	 * @return float
	 * @since 1.0.7.180104
	 */
	public function affwp_recurring_calc_referral_amount($referral_amount, $affiliate_id, $order_amount) {
		$order = wc_get_order($this->recurring_referral_args['reference']);

		if(is_object($order)) {
			$original_amount = $referral_amount;
			$referral_amount = $this->convert_to_awp_currency($original_amount, $order->get_currency());

			// Debug logging
			// @since 1.0.12.200505
			$this->get_logger()->debug(__('Calculating AWP referral amount.', 'wc-aelia-eu-vat-assistant'), array(
				'Original Amount' => $original_amount,
				'Converted Amount' => $referral_amount,
				'Order ID' => $order->get_id(),
				'Order Currency (Source)' => $order->get_currency(),
				'AffiliateWP Currency (Target)' => affwp_get_currency(),
			));
		}

		return $referral_amount;
	}

	/**
	 * Intercepts the creation of a recurring referral, to ensure the conversion of
	 * the referral amounts.
	 *
	 * @param bool create_referral
	 * @param array args
	 * @return bool
	 * @since 1.0.7.180104
	 */
	public function affwp_recurring_create_referral($create_referral, $args) {
		// Debug logging
		// @since 1.0.12.200505
		$this->get_logger()->debug(__('Creating AWP referral.', 'wc-aelia-eu-vat-assistant'), array(
			'Create Referral' => $create_referral,
			'Args' => $args,
		));

		// Store the arguments used to create a recurring referral. They will be
		// used in the "affwp_recurring_calc_referral_amount" filter
		$this->recurring_referral_args = $args;

		// Add the filter to convert the referral amounts
		add_filter('affwp_recurring_calc_referral_amount', array($this, 'affwp_recurring_calc_referral_amount'), 10, 3);

		// We don't need to change the "create referral" flag, we can just return it
		return $create_referral;
	}
}
