<?php
namespace Aelia\WC\CurrencySwitcher\AffiliateWP;
if(!defined('ABSPATH')) { exit; } // Exit if accessed directly

use \Exception;

/**
 * Extends the Aelia Order class, replacing the methods that will return the
 * amounts expected by AffiliateWP.
 *
 * @since 1.0.1.150518
 */
class Order extends \Aelia\WC\Order {
	/**
	 * Returns the instance of the logger used by the plugin.
	 *
	 * @return \Aelia\WC\Logger.
	 * @since 1.0.12.200505
	 */
	protected function get_logger() {
		return Aelia_CS_AffiliateWP::instance()->get_logger();
	}

	/**
	 * Loads an order and returns an instance of this class.
	 *
	 * @param int order_id The ID of the order to load.
	 * @return \Aelia\WC\CurrencySwitcher\AffiliateWP\Order
	 */
	public static function factory($order_id) {
		return new self($order_id);
	}

	/**
	 * Converts an amount from this order's currency to AffiliateWP currency.
	 *
	 * @param float amount The amount to convert.
	 * @return float The amount converted from order currency to AffiliateWP
	 * currency.
	 */
	protected function convert($amount) {
		$amount = apply_filters('wc_aelia_cs_convert',
														$amount,
														// Source currency - This order's currency
														$this->get_currency(),
														// Destination currency - AffiliateWP currency
														affwp_get_currency());
		return apply_filters('wc_aelia_cs_affiliatewp_convert_amount', $amount, $this);
	}

	/**
	 * Converts the order total to AffiliateWP currency.
	 *
	 * @param  string $context View or edit context.
	 * @return float The total for the order, converted to AffiliateWP currency.
	 * @see WC_Order::get_total()
	 * @since 1.1.0.210317
	 */
	public function get_total($context = 'view') {
		$order_total = $this->convert(parent::get_total());

		// Debug logging
		$this->get_logger()->debug(__('Converted order total.', 'wc-aelia-eu-vat-assistant'), array(
			'Order ID' => $this->get_id(),
			'Original Amount' => parent::get_total(),
			'Converted Amount' => $order_total,
			'Order Currency (Source)' => $this->get_currency(),
			'AffiliateWP Currency (Target)' => affwp_get_currency(),
		));

		return apply_filters('wc_aelia_cs_affiliatewp_order_get_total', $order_total, $this);
	}

	/**
	 * Converts the shipping total to AffiliateWP currency.
	 *
	 * @return float The shipping total for the order, converted to AffiliateWP
	 * currency.
	 * @see WC_Order::get_total_shipping()
	 */
	public function get_total_shipping() {
		$total_shipping = $this->convert(parent::get_shipping_total());

		// Debug logging
		// @since 1.0.12.200505
		$this->get_logger()->debug(__('Converted order shipping total.', 'wc-aelia-eu-vat-assistant'), array(
			'Order ID' => $this->get_id(),
			'Original Amount' => parent::get_shipping_total(),
			'Converted Amount' => $total_shipping,
			'Order Currency (Source)' => $this->get_currency(),
			'AffiliateWP Currency (Target)' => affwp_get_currency(),
		));

		return apply_filters('wc_aelia_cs_affiliatewp_order_get_total_shipping', $total_shipping, $this);
	}

	/**
	 * Converts the order items amounts to AffiliateWP currency.
	 *
	 * @param string|array type Types of line items to get.
	 * @return array An array of order items, with their amounts converted to
	 * AffiliateWP currency.
	 * @see WC_Order::get_items()
	 */
	public function get_items($type= 'line_item') {
		$items = parent::get_items($type);

		foreach($items as $item_idx => $product) {
			// Skip products that have already been processed
			// @since 1.0.21.210302
			if(!empty($product->converted_awp_currency)) {
				continue;
			}

			$original_line_total = $product['line_total'];
			$product['line_total'] = $this->convert($original_line_total);

			$original_line_tax = $product['line_tax'];
			$product['line_tax'] = $this->convert($original_line_tax);

			// Debug logging
			// @since 1.0.12.200505
			$this->get_logger()->debug(__('Converted order item total and tax.', 'wc-aelia-eu-vat-assistant'), array(
				'Order ID' => $this->get_id(),
				'Original Line Total' => $original_line_total,
				'Converted Line Total' => $product['line_total'],
				'Original Line Tax' => $original_line_tax,
				'Converted Line Tax' => $product['line_tax'],
				'Order Currency (Source)' => $this->get_currency(),
				'AffiliateWP Currency (Target)' => affwp_get_currency(),
				// @since 1.0.21.210302
				'Order Item Index' => $item_idx,
				'Product' => $product,
			));

			$product->converted_awp_currency = true;

			$items[$item_idx] = $product;
		}
		return apply_filters('wc_aelia_cs_affiliatewp_order_get_items', $items, $this);
	}
}
